//! Knockout Search!
//!
//! A utility to scape the Knockout.chat forums.

use chrono::prelude::*;
use std::env;
use std::fs;

use crate::ko::PopularThread;

pub mod document_structs;
pub mod ko;

/////////////////////////////////////////////////////////////////////////////
/// Takes a vector of Forum Post structs and returns a vector of our own
/// formatted Post struct, and a Vec of assorted UserDocuments.
///
/// Posts are unsorted at this point.
/// The UserDocuments are unsorted and duplicate at this point.
/////////////////////////////////////////////////////////////////////////////
#[allow(dead_code)]
fn build_posts(
    posts: Vec<ko::Post>,
) -> (
    Vec<document_structs::Post>,
    Vec<document_structs::UserDocument>,
) {
    let mut documents: Vec<document_structs::Post> = Vec::with_capacity(posts.len());
    let mut users: Vec<document_structs::UserDocument> = Vec::new();

    for i in posts.iter() {
        //println!("Adding Post: {}\t TN# {}", i.id, i.threadPostNumber);

        let new_user = document_structs::User {
            id: i.user.id,
            user_name: i.user.username.to_owned().unwrap_or_default(), // Changing String ownership from old struct to new.
        };

        let new_post = document_structs::Post {
            id: i.id,
            thread: i.thread,
            content: i.content.to_owned(), // Expensive copy op
            page: i.page,
            creation_time: i.createdAt.parse::<DateTime<Utc>>().unwrap().timestamp() as u64,
            updated_time: 0, // TODO: String to timestamp
            poster: new_user,
            thread_index: i.threadPostNumber,
        };

        documents.push(new_post);
        users.push(document_structs::UserDocument::new(
            i.user.id,
            i.user.username.clone().unwrap_or_default(),
        )); // Extremely inefficient, duplicates users!
    }
    (documents, users)
}

fn flush_thread(thread: ko::Thread) -> Result<(), String> {
    let json = serde_json::to_string(&thread).unwrap();
    fs::create_dir_all("threads").unwrap();
    let path = format!("threads/{}.json", thread.get_id());
    // Write debug file output.
    fs::write(path, json).unwrap();
    Ok(())
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    /////////////////////////////////////////////////////////////////////////
    //      CLI Arguments
    /////////////////////////////////////////////////////////////////////////
    // Gather passed env vars / arguments
    let mut args: Vec<String> = env::args().collect();
    // Check if there's an input.
    if args.len() < 2 {
        println!("No input!");
        println!("Usage: ");
        println!("\t List    1. knockout_search 1 5 16 8 24 (...)");
        println!("\t Range   2. knockout_search 25-800");
        println!("\t Popular 2. knockout_search popular");
        return Ok(());
    };
    // If we're a range instead of a list.
    if args.len() == 2 {
        if args[1].contains(&String::from("-")) {
            let split: Vec<&str> = args[1].split('-').collect();
            let first = split[0]
                .parse::<usize>()
                .expect("First number in range could not be converted to usize!");
            let second = split[1]
                .parse::<usize>()
                .expect("Second number in range could not be converted to usize!");

            args = vec![args[0].to_owned()];
            for i in first..=second {
                args.push(i.to_string());
            }
            println!("Using Range Mode.");
            println!(
                "\tThreads: {f:width$}-{s:width$}\n",
                width = second.to_string().chars().count(),
                f = first,
                s = second,
            )
        }
    }
    if args[1] == "popular" {
        println!("Using Popular Threads Mode.");
        let pops = PopularThread::get().await.unwrap();

        args = vec![args[0].to_owned()];
        print!("\tThreads:");
        for thread in pops.into_iter() {
            print!("{} ", thread.id);
            args.push(thread.id.to_string());
        }
        println!();
    }
    /////////////////////////////////////////////////////////////////////////

    // Go through all of the thread IDs we're given.
    for arg in args.iter().skip(1) {
        println!(); // Extra newline.
        let thread;
        match ko::Thread::new_from_id(arg.parse().unwrap()).await {
            Ok(res) => {
                thread = res;
            }
            Err(reason) => {
                println!("Thread:\t{} SKIPPED! \t Reason: \"{}\"", &arg, reason);
                continue;
            }
        }
        // Write whole thread verbatim.
        flush_thread(thread).unwrap();
    }
    println!("Finished!");
    Ok(())
}
